# A demonstration of the functions that implement W-RF
library(MASS)
data.gen<-function(N=100,nx=15,case="one")
{
  # a function to generate data from
  # different simulation scenarios
  xx<-mvrnorm(n=N,mu=c(0,0),
              Sigma = matrix(c(1,0.2,0.2,1),ncol=2))
  colnames(xx)<-c("x1","x2")
  x1<-xx[,1];x2<-xx[,2]
  for (j in 3:nx)
  {
    assign(paste('x',j,sep=''),rnorm(n=N))
    xx<-eval(parse(text=paste0('cbind(xx,','x',j,')')))
  }
  
  t<-rbinom(n=N,size=1,prob=0.5)
  
  mu<-2+x1+x1^2+x2+x2^2+x3+log(abs(x4))+
    0.1*exp(x5)+sin(x6)+cos(x7)
  
  if (case=="one")
  {
    delta<-3*(x1 +x2) 
  } else if (case=="two")
  {
    delta<-9 - 4*x1^2 - 4*x2^2
  } else if (case=="three")
  {
    delta<-5*(2*as.numeric(x1> -0.5 & x2> -0.5)-1) 
  }
  q<-mu+delta * t
  y<-sapply(q, FUN=function(x){rnorm(n=1,mean=x,sd=1)})
  out<-cbind(xx,t,y)
  return(out)
}

theta<-function(t,y,rule)
{
  # function to estimate
  # the population average outcome under
  # a treatment rule
  sum(y*rule*t)/sum(t)+
    sum(y*(1-rule)*(1-t))/sum((1-t))
}

# generate training dataset and testing dataset
dat.train<-data.gen(N=100,nx=15,'two')
dat.test<-data.gen(N=100000,nx=15,'two')

# using the training dataset to build the model
model<-w.rf(dat=dat.train,ntree=500,x.try=10,min.node.size = 1)
# 1st type Variable importance measure
VI.1<-model[[3]]

# using the testing dataset to predict the rule and
rule<-c.pred.forest(dat=as.matrix(dat.test),forest.obj=model)
# estimate the population average outcome under the derived rule
theta.test<-theta(t=dat.test[,"t"],y=dat.test[,"y"],rule=rule)
# oob estimate of the population average outcome under the derived rule
theta.oob<-theta(t=dat.train[,"t"],
                 y=dat.train[,"y"],rule=as.numeric(model[[2]][,1]))
